// This is a fully documented Typst configuration file.
// If something is unclear, check the Typst docs or file an issue.
// ===============================================================

#import "environments.typ": proposition, lemma, theorem, corollary, definition, remark, example, exercise, question, proof, numbered-equation

// Configuration of the paper.
// ---------------------------

#let configuration(title: none, authors: (), abstract: [], doc) = {
  set document(title: title, author: if authors.len() != 0 {
    for author in authors {
      author.name + " "
    }
  } else {
    "unknown"
  })

  import "constants.typ": *

  set page(paper: "a4", margin: (x: 1.8cm, y: 2.6cm), numbering: "1")

  set text(lang: lang, font: "New Computer Modern", size: 10.5pt)

  // Mathematics.
  set math.mat(delim: "[", row-gap: 0.65em, column-gap: 1em)
  set math.vec(delim: "[", gap: 0.65em)

  // Formatting - similar to LaTeX.
  set par(leading: 0.55em, justify: true)
  set table(stroke: 0.1pt)
  set heading(numbering: "1.")
  set math.equation(supplement: none)

  show sym.lt.eq: $lt.eq.slant$
  show sym.gt.eq: $gt.eq.slant$

  /* Show math figures as normal text. */
  show figure.where(kind: proposition-id): it => it.body
  show       figure.where(kind: lemma-id): it => it.body
  show     figure.where(kind: theorem-id): it => it.body
  show   figure.where(kind: corollary-id): it => it.body
  show  figure.where(kind: definition-id): it => it.body
  show      figure.where(kind: remark-id): it => it.body
  show     figure.where(kind: example-id): it => it.body
  show    figure.where(kind: exercise-id): it => it.body
  show    figure.where(kind: question-id): it => it.body
  show       figure.where(kind: proof-id): it => it.body

  /* Show headings like in amsart. */
  show heading: it => {
    set block(above: 1.8em, below: 1.2em)
    set align(center)
    set text(10pt, weight: "regular")

    if it.body == outline-title {
      block(smallcaps(it.body))
    } else {
      block(counter(heading).display() + " " + smallcaps(it.body))
    }

    /* We want equations to be local to their heading. */
    counter(math.equation).update(0)
  }

  // The paper.
  set align(center)

  text(12pt)[#smallcaps(title)]

  v(1em)

  /* We print authors' information in a grid. */
  let count = authors.len()
  let ncols = calc.min(count, 3)
  grid(columns: (1fr,) * ncols, row-gutter: 24pt, ..authors.map(author => [
    #author.name \
    #author.affiliation \
    #link("mailto:" + author.email)
  ]))

  v(2em)

  if abstract != [] {
    par(justify: false)[
      #smallcaps[#abstract-title] \
      #v(0.15em)
      #abstract
      #v(2em)
    ]
  }

  outline(indent: auto, title: outline-title)

  v(2em)

  set align(left)

  doc
}
